//------------------------------------------------------------------------------
//	$Id: background.js,v 1.32 2021/05/29 00:28:11 wolf Exp $
//------------------------------------------------------------------------------
//	See:	https://www.netzwolf.info/code/addons/genderclown/
//------------------------------------------------------------------------------
//	License: WTF public license (see: http://www.wtfpl.net/)
//------------------------------------------------------------------------------

/*jsl:option explicit*/
/*jsl:import bootstring.js*/

'use strict';

(function() {

	console.debug( 'background.js started @' + new Date().toTimeString() );

	//----------------------------------------------------------------------
	//	settings
	//----------------------------------------------------------------------

	var DEFAULT_SETTINGS = {

		'mode': [ 'default-on', 'default-off', 'on-demand' ][ 0 ],
		'showCounter': true,
		'doInsertClowns': true,
		'clown': '\ud83e\udd21',	// 🤡
		'blockedDomains': [
			'github.com',
			'jscomplete.com',
			'jsfiddle.net',
			'playcode.io',
			'stackexchange.com',
			'stackoverflow.com',
			'w3schools.com',
			'developer.*',
			'developers.*',
			'docs.*'
		].join( '\n' ),
		'doLogging': false,
		'doCleaning': false,
		'doMarkChanges': false,
		'markElement': 'span',
		'markStyle': 'text-decoration:underline wavy blue;background:yellow;color:black',
		'playSound': false,
		'soundVolume': '0.5'
	};

	var settings = {};

	//----------------------------------------------------------------------
	//	build regular exp from a comma/space separated domain list
	//----------------------------------------------------------------------

	var InvalidCharsRegExp = new RegExp( '[^\\s\\p{L}\\p{N}\\p{S}*,.-]+', 'gu' );

	function regexpFromDomains( domains ) {

		//--------------------------------------------------------------
		//	split string containg space separated domain names
		//--------------------------------------------------------------

		var domainList = (domains || '').trim().toLowerCase().
			replace( InvalidCharsRegExp, 'x' ).split( /[,\s]+/ );

		if( !domainList.length ) return new RegExp( '^\\b$' );

		//--------------------------------------------------------------
		//	create regexp pattern for each domain
		//--------------------------------------------------------------

		var patternList = [];

		for( var i = 0 ; i < domainList.length ; ++i ) {

			//-----------------------------------------------------
			//	punicode encode domain parts
			//-----------------------------------------------------

			var fields = domainList[ i ].split( '.' );
			var encodedFields = [];

			for( var j = 0 ; j < fields.length ; ++j ) {

				encodedFields.push(
				BootString.punycodeEncode( fields[ j ] ));
			}

			patternList.push( encodedFields.join( '\\.' ));
		}

		//--------------------------------------------------------------
		//	match any domain from patternList including subdomains
		//--------------------------------------------------------------
		//	^   (.*\.)?   ( pat1 | pat2 | .. patn )   $
		//--------------------------------------------------------------

		return new RegExp(
			'^(.*\\.)?(' +
				patternList.join( '|' ).replace( /\*/g, '.*' )+
			')$' );
	}

	//----------------------------------------------------------------------
	//	regular expression matching blocked domains
	//----------------------------------------------------------------------

	var regExpBlockedDomains = null;

	function updateRegExps( settings ) {

		regExpBlockedDomains =
			regexpFromDomains( settings.blockedDomains );
	}

	//----------------------------------------------------------------------
	//	read settings from storage.local using DEFAULT_SETTINGS
	//----------------------------------------------------------------------

	function readSettings() {

		chrome.storage.local.get( function( result ) {

			//------------------------------------------------------
			//	add default values if needed
			//------------------------------------------------------

			var updates = {};

			for( var key in DEFAULT_SETTINGS ) {

				if( result[ key ] === undefined ) {

					result [ key ] = DEFAULT_SETTINGS[ key ];
				}
			}

			//------------------------------------------------------
			//	update settings and icon
			//------------------------------------------------------

			settings = result;

			//------------------------------------------------------
			//	update hostlist regular expression
			//------------------------------------------------------

			updateRegExps( settings );
		});
	}

	//----------------------------------------------------------------------
	//	inititial read settings
	//----------------------------------------------------------------------

	readSettings();

	//----------------------------------------------------------------------
	//	read again after storage changes
	//----------------------------------------------------------------------

	chrome.storage.onChanged.addListener( readSettings );

	//----------------------------------------------------------------------
	//	editing on/off state per tab
	//----------------------------------------------------------------------
	// (!)	TabStates will grow slowly but infinitely until browser close
	//----------------------------------------------------------------------

	var TabPermState = {};
	var TabTempState = {};

	//----------------------------------------------------------------------
	//	init tabstates depending on allowing/blocking of domain
	//----------------------------------------------------------------------

	function initTabState( tabId, hostname ) {

		//--------------------------------------------------------------
		//	blocked domains use page local state
		//--------------------------------------------------------------

		if( regExpBlockedDomains &&
			regExpBlockedDomains.test( hostname.toLowerCase() )) {

			TabTempState[ tabId ] = false;
			return;
		}

		//--------------------------------------------------------------
		//	mode = on demand → init to false
		//--------------------------------------------------------------

		if( settings.mode === 'on-demand' ) {

			TabTempState[ tabId ] = false;
			return;
		}

		//--------------------------------------------------------------
		//	mode = default-on/off: init if not yet initted
		//--------------------------------------------------------------

		if( typeof( TabTempState[ tabId ]) != 'boolean' ) {

			TabTempState[ tabId ] = settings.mode === 'default-on';
			return;
		}
	}

	//----------------------------------------------------------------------
	//	prepare message settings and set action=activate if active
	//	sets icon to colored if active and gray if not active
	//----------------------------------------------------------------------

	function setIconAndPrepareMessage( tabId ) {

		var message = { 'settings': JSON.stringify( settings ) };

		var activate = TabTempState[ tabId ] === null ?
			TabPermState[ tabId ] : TabTempState[ tabId ];

		if( activate ) {

			message[ 'action' ] = 'activate';

			chrome.browserAction.setIcon( {

				tabId: tabId,
				path: 'icons/clown-128x128t.png'
			});

		} else {
			chrome.browserAction.setIcon( {

				tabId: tabId,
				path: 'icons/clown-gray-128x128t.png'
			});
		}

		return message;
	}

	//----------------------------------------------------------------------
	//	process click on addon icon in browser tab
	//----------------------------------------------------------------------

	chrome.browserAction.onClicked.addListener( function( tab ) {

		var tabId = tab.id;

		//--------------------------------------------------------------
		//	process click changing tabstate
		//--------------------------------------------------------------

		if( TabTempState[ tabId ] !== null ) {

			TabTempState[ tabId ] = !TabTempState[ tabId ];
		} else {
			TabPermState[ tabId ] = !TabPermState[ tabId ];
		}

		//--------------------------------------------------------------
		//	set icon and send message
		//--------------------------------------------------------------

		var message = setIconAndPrepareMessage( tabId );
		chrome.tabs.sendMessage( tabId, message );
        });

	//----------------------------------------------------------------------
	//	sound
	//----------------------------------------------------------------------

	var lastSoundStartTime = 0;

	function playSound() {

		if( !settings.playSound ) return;

		var now = new Date().getTime();
		if( now < lastSoundStartTime + 333 ) return;

		lastSoundStartTime = now;

		var player = new Audio( 'sounds/pop.mp3' );
		player.volume = parseFloat( settings.soundVolume ) || 0.5;
		player.play();
		player = null;
	}

	//----------------------------------------------------------------------
	//	process message from content-script or options-script
	//----------------------------------------------------------------------

	chrome.runtime.onMessage.addListener( function( request, sender, sendResponse ) {

		var tabId = sender.tab && sender.tab.id || null;

		switch( request.action ) {

		case 'initSettings':

			initTabState( tabId, request.hostname );
			sendResponse( setIconAndPrepareMessage( tabId ));
			break;

		case 'readSettings':

			sendResponse( { 'settings': JSON.stringify( settings ) } );
			break;

		case 'writeSettings':

			settings = JSON.parse( request.settings );
			updateRegExps( settings );
			chrome.storage.local.set( settings );
			break;

		case 'setMessage':

			var newIconTitle = request.active ?
				chrome.i18n.getMessage( 'titleReplacementActive' ) :
			settings.mode !== 'on-demand' ?
				chrome.i18n.getMessage( 'titleReplacementNotActive' ) :
				chrome.i18n.getMessage( 'titleReplacementOnDemand' );

			chrome.browserAction.setTitle( {
				tabId: tabId,
				title: newIconTitle
			});
			break;

		case 'setCount':

			if( request.count ) playSound();

			chrome.browserAction.setBadgeText( {
				tabId: tabId,
				text: request.count ? '' + request.count : ''
			});
			break;

		default:
			break;
                }
	});
})();

//------------------------------------------------------------------------------
//	$Id: background.js,v 1.32 2021/05/29 00:28:11 wolf Exp $
//------------------------------------------------------------------------------
