//------------------------------------------------------------------------------
//	$Id: bootstring.js,v 1.2 2021/02/10 19:53:06 wolf Exp $
//------------------------------------------------------------------------------
//	See:	https://www.netzwolf.info/code/addons/genderclown/
//------------------------------------------------------------------------------
//	License: WTF public license (see: http://www.wtfpl.net/)
//------------------------------------------------------------------------------

/*jsl:option explicit*/

'use strict';

var BootString = {

//------------------------------------------------------------------------------
//	bootstring encoding
//------------------------------------------------------------------------------

encode: function( unencoded, params ) {

	//----------------------------------------------------------------------
	//	convert chars to codes, gather basic chars
	//----------------------------------------------------------------------

	var lastCode = params.initial_n;

	var codes  = [];
	var output = [];

	for( var i = 0 ; i < unencoded.length ; ++i ) {

		var code = unencoded.charCodeAt( i );
		codes.push( code );

		if( code < lastCode ) output.push( unencoded.charAt( i ) );
	}

	var nCharsEncoded = output.length;

	//----------------------------------------------------------------------
	//	no encoding needed
	//----------------------------------------------------------------------

	if( unencoded.length == nCharsEncoded ) return unencoded;

	//----------------------------------------------------------------------
	//	separate une
	//----------------------------------------------------------------------

	if( nCharsEncoded >= 1 ) output.push( params.delimiter );

	//----------------------------------------------------------------------
	//	overview
	//----------------------------------------------------------------------

	var bias  = params.initial_bias;
	var scale = params.damp;
	var delta = 0;

	for (;;) {

		//--------------------------------------------------------------
		//	find lowest code not yet encoded
		//--------------------------------------------------------------

		var nextCode = 0;

		for( i = 0 ; i < codes.length ; ++i ) {

			code = codes[ i ];
			if( code < lastCode ) continue;

			if( !nextCode || code < nextCode ) nextCode = code;
		}

		if( !nextCode ) break;

		//--------------------------------------------------------------
		//	skip unused codepoints
		//--------------------------------------------------------------

		delta += (nextCode - lastCode) * (nCharsEncoded + 1);

		//--------------------------------------------------------------
		//	process hits
		//--------------------------------------------------------------

		for( i = 0 ; i < codes.length ; ++i ) {

			code = codes[ i ];

			//------------------------------------------------------
			//	not encoded at this iteration
			//------------------------------------------------------

			if( code > nextCode ) continue;

			//------------------------------------------------------
			//	encode one character
			//------------------------------------------------------

			if( code == nextCode ) {

				//----------------------------------------------
				//	variable length integer encoding
				//----------------------------------------------

				var quot = delta;

				for( var k = 1 ;; ++k ) {

					var thres = k * params.base - bias;
					if( thres < params.tmin ) thres = params.tmin;
					if( thres > params.tmax ) thres = params.tmax;

					if( quot < thres ) break;

					var digit = ((quot - thres) % (params.base - thres)) + thres;
					quot = Math.floor( (quot - thres) / (params.base - thres) );

					if( digit < 0 || digit >= params.base ) throw 'Cannot happen';

					output.push( params.encodeChars.charAt( digit ));
				}

				if( quot < 0 || quot >= params.base ) throw 'Cannot happen';
				output.push( params.encodeChars.charAt( quot ));

				++nCharsEncoded;

				//----------------------------------------------
				//	Bias adaption
				//----------------------------------------------

				bias = 0;

				delta  = Math.floor( delta / scale         );
				delta += Math.floor( delta / nCharsEncoded );
				scale  = 2;

				while( delta > Math.floor( ((params.base - params.tmin) * params.tmax) / 2) ) {

       					delta = Math.floor( delta / (params.base - params.tmin) );
					bias += params.base;
				}

				bias += Math.floor( ((params.base - params.tmin + 1) * delta) / (delta + params.skew) );

				//----------------------------------------------
				//	Reset counter
				//----------------------------------------------

				delta = 0;
			} else {
				++delta;
			}
		}

		++delta;
		lastCode = nextCode + 1;
	}

	return params.prefix + output.join( '' );
},

//------------------------------------------------------------------------------
//	test bootstring encoding
//------------------------------------------------------------------------------

verifyEncoding: function( refRaw, refEncoded, params ) {

	var encoded = this.encode( refRaw, params );

	if( encoded !== refEncoded ) throw 'Encoding [ ' + refRaw +
		' ] returns [ ' + encoded +
		' ], but [ ' +refEncoded + ' ] expected.';
},

//------------------------------------------------------------------------------
//	puny parameters for BootString
//------------------------------------------------------------------------------

PUNYCODE_PARAMS: {
	'msg_prefix':  'Punycode',
	'base':         36,
	'tmin':          1,
	'tmax':         26,
	'skew':         38,
	'damp':        700,
	'bias':         72,
	'initial_n':   128,
	'initial_bias': 72,
	'prefix':       'xn--',
	'delimiter':    '-',
	'encodeChars':  'abcdefghijklmnopqrstuvwxyz0123456789',
	'decodeChars': {
		'a':  0, 'b':  1, 'c':  2, 'd':  3, 'e':  4, 'f':  5,
		'g':  6, 'h':  7, 'i':  8, 'j':  9, 'k': 10, 'l': 11,
		'm': 12, 'n': 13, 'o': 14, 'p': 15, 'q': 16, 'r': 17,
		's': 18, 't': 19, 'u': 20, 'v': 21, 'w': 22, 'x': 23,
		'y': 24, 'z': 25, '0': 26, '1': 27, '2': 28, '3': 29,
		'4': 30, '5': 31, '6': 32, '7': 33, '8': 34, '9': 35
	}
},

//------------------------------------------------------------------------------
//	punycode encoding
//------------------------------------------------------------------------------

punycodeEncode: function( unencoded ) {

	return this.encode( unencoded, this.PUNYCODE_PARAMS );
},

//------------------------------------------------------------------------------
//	test punicode bootstring encoding
//------------------------------------------------------------------------------

verifyPunycodeEncoding: function( refRaw, refEncoded ) {

	var encoded = this.punycodeEncode( refRaw );

	if( encoded !== refEncoded ) throw 'Encoding [ ' + refRaw +
		' ] returns [ ' + encoded +
		' ], but [ ' +refEncoded + ' ] expected.';
},

//------------------------------------------------------------------------------
//	test "punycodeEncode()" using a set of examples
//------------------------------------------------------------------------------

testPunycodeEncoding: function() {

	console.log( 'Test Punycode-Encoding...' );

	this.verifyPunycodeEncoding( 'bücher'  , 'xn--bcher-kva'  );
	this.verifyPunycodeEncoding( 'büücher' , 'xn--bcher-kvaa' );
	this.verifyPunycodeEncoding( 'bücüher' , 'xn--bcher-kvab' );
	this.verifyPunycodeEncoding( 'köln'    , 'xn--kln-sna'    );
	this.verifyPunycodeEncoding( 'ùúûü'    , 'xn--qdacde'     );
	this.verifyPunycodeEncoding( 'äöü'     , 'xn--4ca0bs'     );

	this.verifyPunycodeEncoding(
		'\u4ED6\u4EEC\u4E3A\u4EC0\u4E48\u4E0D\u8BF4\u4E2D\u6587',
		'xn--ihqwcrb4cv8a8dqg056pqjye' );

	this.verifyPunycodeEncoding(
		'\u0033\u5E74\u0042\u7D44\u91D1\u516B\u5148\u751F',
		'xn--3B-ww4c5e180e575a65lsy2b' );

	this.verifyPunycodeEncoding(
		'點看點看點看點看點看點看',
		'xn--c1yaaaaa3742ebabbbb' );

	this.verifyPunycodeEncoding(
		'點點點點點點點點看看看看看看看看',
		'xn--c1yaaaaaaa1682gaaaaaaa' );

	console.log( 'Test Punycode-Encoding... Ok' );
}

};

//------------------------------------------------------------------------------
//	run tests if not in browser
//------------------------------------------------------------------------------

if( typeof( navigator ) === 'undefined' ) BootString.testPunycodeEncoding();

//------------------------------------------------------------------------------
//	$Id: bootstring.js,v 1.2 2021/02/10 19:53:06 wolf Exp $
//------------------------------------------------------------------------------
