//------------------------------------------------------------------------------
//	$Id: page-modifier.js,v 1.16 2021/02/10 19:21:38 wolf Exp $
//------------------------------------------------------------------------------
//	See:	https://www.netzwolf.info/code/addons/genderclown/
//------------------------------------------------------------------------------
//	License: WTF public license (see: http://www.wtfpl.net/)
//------------------------------------------------------------------------------

/*jsl:option explicit*/
/*jsl:import node-modifier.js*/

'use strict';

(function() {

	console.debug( 'page-modifier.js started @' + new Date().toTimeString() );

	//----------------------------------------------------------------------
	//	settings
	//----------------------------------------------------------------------

	var settings = {};
	var active   = false;

	//----------------------------------------------------------------------
	//	change counter
	//----------------------------------------------------------------------

	var nChanges = 0;
	var nChangesSent = null;

	function sendChangeCount() {

		if( !settings.showCounter || nChanges === nChangesSent ) return;

		chrome.runtime.sendMessage( {
       			'action': 'setCount',
			'count' : nChanges || ''
		});

		nChangesSent = nChanges;
	}

	//----------------------------------------------------------------------
	//	Select nodes to be processed
	//----------------------------------------------------------------------

	function checkNode( node ) {

		//--------------------------------------------------------------
		//	parent
		//--------------------------------------------------------------

		if( !node.parentElement ) return NodeFilter.FILTER_ACCEPT;

		//--------------------------------------------------------------
		//	protect some html elements
		//--------------------------------------------------------------

		switch( node.parentNode.nodeName ) {

		case 'NOSCRIPT':
		case 'SCRIPT':
		case 'STYLE':
		case 'TEXTAREA':

			return NodeFilter.FILTER_REJECT;

		default:
			break;
		}

		//--------------------------------------------------------------
		//	protect component of editable areas
		//--------------------------------------------------------------

		if( node.parentElement.getAttribute( 'role' ) == 'textbox' ) {

	                           	return NodeFilter.FILTER_REJECT;
		}

		if( node.parentElement.isContentEditable ) {

	                           	return NodeFilter.FILTER_REJECT;
		}

		//--------------------------------------------------------------
		//	protect marked areas
		//--------------------------------------------------------------

		if( node.parentNode.className == 'no-genderclown' ) {

			return NodeFilter.FILTER_REJECT;
		}

		//--------------------------------------------------------------
		//	accept
		//--------------------------------------------------------------

		return NodeFilter.FILTER_ACCEPT;
	}

	//----------------------------------------------------------------------
	//	process subtree under root node
	//----------------------------------------------------------------------

	function processSubtree( rootElement ) {

		if( !rootElement.textContent.length ) return;

		var startTime = new Date().getTime();

		//--------------------------------------------------------------
		//	use coarse filter to avoid unnecessary processing 
		//--------------------------------------------------------------

		var mode = NodeModifier.coarseFilter(
				rootElement.textContent, settings );

		if( !mode ) {

			if( settings.doLogging ) {

				console.info( 'genderclown: ' +
					rootElement.textContent.length +
					' chars in ' +
					( new Date().getTime() - startTime ) +
					' ms.' );
			}

			return;
		}

		//--------------------------------------------------------------
		//	process text nodes of subtree
		//--------------------------------------------------------------

		var savedNChanges = nChanges;

		var walker = document.createTreeWalker( rootElement,
			NodeFilter.SHOW_TEXT, { acceptNode: checkNode } );

		for (;;) {

			//------------------------------------------------------
			//	get node
			//------------------------------------------------------

			var node = walker.nextNode();
			if (!node) break;

			//------------------------------------------------------
			//	modify node data
			//------------------------------------------------------

			nChanges += NodeModifier.processNode(
							mode, node, settings );
		}

		if( settings.doLogging ) {

			console.info( 'genderclown: ' +
				( nChanges - savedNChanges ) +
				' replacements on ' +
				rootElement.textContent.length + ' chars in ' +
				( new Date().getTime() - startTime ) +
				' ms.' );
		}
	}

	//----------------------------------------------------------------------
	//	process document and 
	//----------------------------------------------------------------------

	function processDocument() {

		sendChangeCount();

		processSubtree( document.documentElement, settings );

		sendChangeCount();

		//--------------------------------------------------------------
		//	wait for and process dynamically added elements
		//--------------------------------------------------------------

		var observer = new MutationObserver( function ( mutations ) {

			for( var m = 0 ; m < mutations.length ; ++m ) {

				var mutation = mutations[ m ];
				if( mutation.type !== 'childList' ) continue;

				for( var n = 0 ; n < mutation.addedNodes.length ; ++n ) {

					processSubtree(
						mutation.addedNodes[ n ], settings );
				}
			}

			sendChangeCount();
		});

		observer.observe( document.documentElement, {
			childList: true,
			subtree: true
		});
	}

	//----------------------------------------------------------------------
	//	process message from background-scripts
	//----------------------------------------------------------------------

	function processMessage( message ) {

		if( message.settings ) {

    			settings = JSON.parse( message.settings );
		}

		if( message.action === 'activate' && !active ) {

			processDocument();
			active = true;
		}

		chrome.runtime.sendMessage(
			{ 'action': 'setMessage', 'active': active } );
	}

	//----------------------------------------------------------------------
	//	add message listener
	//----------------------------------------------------------------------

	chrome.runtime.onMessage.addListener( processMessage );

	//----------------------------------------------------------------------
	//	initial request settings
	//----------------------------------------------------------------------

	var initMessage = {
		'action': 'initSettings',
		'protocol': document.location.protocol,
		'hostname': document.location.hostname
	};

	chrome.runtime.sendMessage( initMessage, processMessage );
})();

//------------------------------------------------------------------------------
//	$Id: page-modifier.js,v 1.16 2021/02/10 19:21:38 wolf Exp $
//------------------------------------------------------------------------------
